/*
****************************************************************************
****************************************************************************
**                                                                        **
**                XXXX     X    X         X XXXXXX X                      **
**               X    X   X X   X    X    X X      X   X                  **
**               X       X   X  X   X X   X X      X  X                   **
**               X   XX XXXXXXX X  X   X  X XXXX   X X                    **
**               X    X X     X X X     X X X      XX                     **
**               X    X X     X XX       XX X      X                      **
**                XXXX  X     X X         X XXXXXX XXXXXX                 **
**                                                                        **
**                                                                        **
**                           BIALYSTOK 2020                               **
**                                                                        **
**                             UDP example                                **
**                                                                        **
**                     Mikrokontroler ARM: LPC2378                        **
**                                                                        **
**                     Wersja z dnia : 2020.11.26                         **
**                     Napisal : Andrzej Pawluczuk                        **
**                                                                        **
****************************************************************************
****************************************************************************
*/

#include <LPC23xx.H>
#include "system.h"
#include "types.h"
#include "systime.h"
#include "irqvectors.h"
#include "serialconst.h"
#include "uart0.h"
#include "strings.h"
#include "emacdriver.h"
#include "ipservice.h"
#include "udpservice.h"
#include "trace.h"

#define IRQCallCntLimit                 500

typedef struct {
                 EMACInstanceRecT *          EMACLayer ;
                 IPLayerInstanceRecT *       IPLayer ;
                 PtrUDPLayerInstanceRecT     UDPLayer ;
                 PtrUDPSocketDescriptionRecT UDPSocket1 ;
                 PtrUDPSocketDescriptionRecT UDPSocket2 ;
                 PtrUDPSocketDescriptionRecT UDPSocket3 ;
                 PtrUDPSocketDescriptionRecT UDPSocket4 ;
               } UDPExampleInstanceRecT ;
typedef UDPExampleInstanceRecT * PtrUDPExampleInstanceRecT ;

static UDPExampleInstanceRecT UDPDemoInstance ; ;
static ULONG IRQCallCounter ;

#define NetBufferSize                256

#define MachineIPAddress        0xC0A80037
//                              ^^^ 192.168.0.55
#define MachineIPSubnet         0xFFFFFF00
//                              ^^^ 255.255.255.0
#define MachineGWIPAddress      0xC0A800FE
//                              ^^^ 192.168.0.254
#define UDPPortNumber           20001

UCHAR MachineMACAddr [ 6 ]     = { 0x08 , 0x14 , 0x16 , 0x22 , 0x28 , 0x2D } ;

static UCHAR NoUDPData [ ]     = "Pusty pakiet UDP" ;
static UCHAR Error1 [ ]        = "\r\nProblem z tworzeniem gniazdka\r\n" ;
static UCHAR Error2 [ ]        = "\r\nProblem z nadaniem numeru gniazdku\r\n" ;

static __irq void T1_IRQHandler ( void )
{
  /*-------------------------------------------------------------------------*/
  IRQCallCounter ++ ;
  if ( IRQCallCounter >= IRQCallCntLimit )
  {
    IRQCallCounter = 0 ;
  } /* if */ ;
  T1IR = 1 ;
  VICVectAddr = 0 ;
} /* T1_IRQHandler */


static void HardwareInit ( void ) 
{
  /*-------------------------------------------------------------------------*/
  PINSEL10 = 0 ;
  T1MR0 = ( PCLK / 1000 ) - 1 ;
  T1MCR = 3 ;
  T1TCR = 1 ;
  VICVectAddr5 = ( unsigned long ) T1_IRQHandler ;
  VICVectCntl5 = 15 ;
  VICIntEnable = ( 1 << TIMER1_Interrupt ) ;
  UART0InitEnvir ( ) ;
} /* HardwareInit */


static void StartNet ( PtrUDPExampleInstanceRecT Instance )
{
  EtherSpeedMode EthMode ;
  /*-------------------------------------------------------------------------*/
  EthMode = Mode_AutoNeg ;
//  EthMode = Mode_10MBIT ;
//  EthMode = Mode_100MBIT ;
  Instance -> EMACLayer = InitPHYLayerInstance ( MachineMACAddr , EthMode ) ;
  Instance -> IPLayer = InitIPLayerInstance ( Instance -> EMACLayer ,
                                              ( ULONG ) MachineIPAddress ,
                                              ( ULONG ) MachineIPSubnet ) ;
  Instance -> IPLayer -> MyDefaultGWIPAddress = ( ULONG ) MachineGWIPAddress ;
  Instance -> UDPLayer = StandardAddUDPInstance ( Instance -> IPLayer ) ;
} /* StartNet */


UCHAR Hello1 [ ]      = "\r\nHello: serwer UDP\r\n" ;
UCHAR Hello2 [ ]      = "\r\nMoj adres IP=" ;
UCHAR Hello3 [ ]      = "\r\nMaska podsieci=" ;
UCHAR Hello4 [ ]      = "\r\nAdres bramy domyslnej=" ;
UCHAR Hello6 [ ]      = "\r\nAdres MAC=" ;
UCHAR Hello7 [ ]      = "\r\n************************\r\n" ;

static void HelloMessage ( void )
{
  /*-------------------------------------------------------------------------*/
  UART0SendString ( Hello1 ) ;
  UART0SendString ( Hello2 ) ;
  TraceIP ( MachineIPAddress ) ;
  UART0SendString ( Hello3 ) ;
  TraceIP ( MachineIPSubnet ) ;
  UART0SendString ( Hello4 ) ;
  TraceIP ( MachineGWIPAddress ) ;
  UART0SendString ( Hello6 ) ;
  TraceMAC ( MachineMACAddr ) ;
  UART0SendString ( Hello7 ) ;
} /* HelloMessage */


static void SoftwareInit ( void )
{
  /*-------------------------------------------------------------------------*/
  IRQCallCounter = 0 ;
  UDPDemoInstance . EMACLayer = NIL ;
  UDPDemoInstance . IPLayer = NIL ;
  UDPDemoInstance . UDPLayer = NIL ;
  UDPDemoInstance . UDPSocket1 = NIL ;
  UDPDemoInstance . UDPSocket2 = NIL ;
  UDPDemoInstance . UDPSocket3 = NIL ;
  UDPDemoInstance . UDPSocket4 = NIL ;
} /* SoftwareInit */


static void SendDataBuffer ( void * UDPDataBuffer , 
	                           USHORT UDPDataSize ) 
{
  UCHAR * ChPtr ;
  USHORT Loop ;
  /*-------------------------------------------------------------------------*/
  if ( UDPDataSize == 0 )
  {
    UART0SendString ( NoUDPData ) ;
  } /* if ... */
  else
  {
    ChPtr = ( UCHAR * ) UDPDataBuffer ;
    for ( Loop = 0 ; Loop < UDPDataSize ; Loop ++ )
    {
      if ( ( ( * ChPtr ) >= 0x20 ) && ( ( * ChPtr ) < 0x80 ) )
        UART0Send ( * ChPtr ) ;
      else
        UART0Send ( '.' ) ;
      ChPtr ++ ;
    } /* for */ ;
  } /* if ... else */ ;
  UART0Send ( '\r' ) ;
  UART0Send ( '\n' ) ;
} /* SendDataBuffer */

	
static void UDPStream1Service ( PtrUDPSocketDescriptionRecT UDPSocket ,
                                ULONG                       SenderIPAddress ,
                                USHORT                      SenderPortNo ,
                                ULONG                       DestIPAddress ,
                                USHORT                      DestPortNo ,
                                void *                      UDPDataBuffer ,
                                USHORT                      UDPDataSize )
{
  /*-------------------------------------------------------------------------*/
  UART0SendString ( ( UCHAR * ) "UART0: " ) ;
  SendDataBuffer ( UDPDataBuffer , UDPDataSize ) ;
} /* UDPStream1Service */


static void UDPStream2Service ( PtrUDPSocketDescriptionRecT UDPSocket ,
                                ULONG                       SenderIPAddress ,
                                USHORT                      SenderPortNo ,
                                ULONG                       DestIPAddress ,
                                USHORT                      DestPortNo ,
                                void *                      UDPDataBuffer ,
                                USHORT                      UDPDataSize )
{
  /*-------------------------------------------------------------------------*/
  UART0SendString ( ( UCHAR * ) "UART1: " ) ;
  SendDataBuffer ( UDPDataBuffer , UDPDataSize ) ;
} /* UDPStream2Service */


static void UDPStream3Service ( PtrUDPSocketDescriptionRecT UDPSocket ,
                                ULONG                       SenderIPAddress ,
                                USHORT                      SenderPortNo ,
                                ULONG                       DestIPAddress ,
                                USHORT                      DestPortNo ,
                                void *                      UDPDataBuffer ,
                                USHORT                      UDPDataSize )
{
  /*-------------------------------------------------------------------------*/
  UART0SendString ( ( UCHAR * ) "UART2: " ) ;
  SendDataBuffer ( UDPDataBuffer , UDPDataSize ) ;
} /* UDPStream3Service */


static void UDPStream4Service ( PtrUDPSocketDescriptionRecT UDPSocket ,
                                ULONG                       SenderIPAddress ,
                                USHORT                      SenderPortNo ,
                                ULONG                       DestIPAddress ,
                                USHORT                      DestPortNo ,
                                void *                      UDPDataBuffer ,
                                USHORT                      UDPDataSize )
{
  /*-------------------------------------------------------------------------*/
  UART0SendString ( ( UCHAR * ) "UART3: " ) ;
  SendDataBuffer ( UDPDataBuffer , UDPDataSize ) ;
} /* UDPStream4Service */


static void Halt ( UCHAR * Message )
{
  /*-------------------------------------------------------------------------*/
  UART0SendString ( Message ) ;
  for ( ; ; )
  {
    NOP ( ) ;
  } /* loop */ ;
} /* Halt */


int main ( void )
{
  USHORT BindStatus ;
  /*-------------------------------------------------------------------------*/
  SysInit ( ) ;
  InitSysTime ( ) ;
  HardwareInit ( )  ;
  SoftwareInit ( ) ;
  UART0HardwInit( FOSC , N81Mode , 9600 ) ;
  HelloMessage ( ) ;
  StartNet ( & UDPDemoInstance ) ;
  UDPDemoInstance . UDPSocket1 =  CreateUDPSocket ( UDPDemoInstance . UDPLayer ) ;
  if ( ! UDPDemoInstance . UDPSocket1 )
    Halt ( Error1 ) ;
  UDPDemoInstance . UDPSocket2 =  CreateUDPSocket ( UDPDemoInstance . UDPLayer ) ;
  if ( ! UDPDemoInstance . UDPSocket2 )
    Halt ( Error1 ) ;
  UDPDemoInstance . UDPSocket3 =  CreateUDPSocket ( UDPDemoInstance . UDPLayer ) ;
  if ( ! UDPDemoInstance . UDPSocket3 )
    Halt ( Error1 ) ;
  UDPDemoInstance . UDPSocket4 =  CreateUDPSocket ( UDPDemoInstance . UDPLayer ) ;
  if ( ! UDPDemoInstance . UDPSocket4 )
    Halt ( Error1 ) ;
  BindStatus = BindUDPSocket ( UDPDemoInstance . UDPSocket1 , UDPPortNumber ) ;
  if ( BindStatus != UDP_BindPortOK )
  {
    Halt ( Error2 ) ;
  } /* if */ ;
  BindStatus = BindUDPSocket ( UDPDemoInstance . UDPSocket2 , UDPPortNumber + 1 ) ;
  if ( BindStatus != UDP_BindPortOK )
  {
    Halt ( Error2 ) ;
  } /* if */ ;
  BindStatus = BindUDPSocket ( UDPDemoInstance . UDPSocket3 , UDPPortNumber + 2 ) ;
  if ( BindStatus != UDP_BindPortOK )
  {
    Halt ( Error2 ) ;
  } /* if */ ;
  BindStatus = BindUDPSocket ( UDPDemoInstance . UDPSocket4 , UDPPortNumber + 3 ) ;
  if ( BindStatus != UDP_BindPortOK )
  {
    Halt ( Error2 ) ;
  } /* if */ ;
  SetUDPListen ( UDPDemoInstance . UDPSocket1 , UDPStream1Service ) ;
  SetUDPListen ( UDPDemoInstance . UDPSocket2 , UDPStream2Service ) ;
  SetUDPListen ( UDPDemoInstance . UDPSocket3 , UDPStream3Service ) ;
  SetUDPListen ( UDPDemoInstance . UDPSocket4 , UDPStream4Service ) ;
  for ( ; ; )
  {
    SysTimerPool ( ) ;
    UDPDemoInstance . IPLayer -> IPLayerPoolService ( UDPDemoInstance . IPLayer ) ;
  } /* loop */ ;
} /* main */

